@@ -1,14 +1,21 @@
-# Copyright (c) 2008, 2009, 2010 Jonathan M. Lange. See LICENSE for details.
+# Copyright (c) 2008-2011 testtools developers. See LICENSE for details.
 
 """Extensions to the standard Python unittest library."""
 
 __all__ = [
     'clone_test_with_new_id',
     'ConcurrentTestSuite',
+    'ErrorHolder',
+    'ExpectedException',
     'ExtendedToOriginalDecorator',
+    'FixtureSuite',
     'iterate_tests',
+    'MultipleExceptions',
     'MultiTestResult',
+    'PlaceHolder',
+    'run_test_with',
     'TestCase',
+    'TestCommand',
     'TestResult',
     'TextTestResult',
     'RunTest',
@@ -16,17 +23,31 @@ __all__ = [
     'skipIf',
     'skipUnless',
     'ThreadsafeForwardingResult',
+    'try_import',
+    'try_imports',
     ]
 
+from testtools.helpers import (
+    try_import,
+    try_imports,
+    )
 from testtools.matchers import (
     Matcher,
     )
+# Shut up, pyflakes. We are importing for documentation, not for namespacing.
+Matcher
+
 from testtools.runtest import (
+    MultipleExceptions,
     RunTest,
     )
 from testtools.testcase import (
+    ErrorHolder,
+    ExpectedException,
+    PlaceHolder,
     TestCase,
     clone_test_with_new_id,
+    run_test_with,
     skip,
     skipIf,
     skipUnless,
@@ -40,8 +61,12 @@ from testtools.testresult import (
     )
 from testtools.testsuite import (
     ConcurrentTestSuite,
+    FixtureSuite,
     iterate_tests,
     )
+from testtools.distutilscmd import (
+    TestCommand,
+)
 
 # same format as sys.version_info: "A tuple containing the five components of
 # the version number: major, minor, micro, releaselevel, and serial. All
@@ -55,4 +80,4 @@ from testtools.testsuite import (
 # If the releaselevel is 'final', then the tarball will be major.minor.micro.
 # Otherwise it is major.minor.micro~$(revno).
 
-__version__ = (0, 9, 4, 'final', 0)
+__version__ = (0, 9, 14, 'final', 0)
