@@ -1,17 +1,25 @@
-# Copyright (c) 2008 Jonathan M. Lange. See LICENSE for details.
+# Copyright (c) 2008-2012 testtools developers. See LICENSE for details.
 
 """Helpers for tests."""
 
-import sys
-
-__metaclass__ = type
 __all__ = [
     'LoggingResult',
     ]
 
+import sys
+
 from testtools import TestResult
+from testtools.helpers import (
+    safe_hasattr,
+    )
+from testtools.content import TracebackContent
+from testtools import runtest
 
 
+# Importing to preserve compatibility.
+safe_hasattr
+
+# GZ 2010-08-12: Don't do this, pointlessly creates an exc_info cycle
 try:
     raise Exception
 except Exception:
@@ -62,6 +70,36 @@ class LoggingResult(TestResult):
         self._events.append('done')
         super(LoggingResult, self).done()
 
-# Note, the following three classes are different to LoggingResult by
-# being fully defined exact matches rather than supersets.
-from testtools.testresult.doubles import *
+    def tags(self, new_tags, gone_tags):
+        self._events.append(('tags', new_tags, gone_tags))
+        super(LoggingResult, self).tags(new_tags, gone_tags)
+
+    def time(self, a_datetime):
+        self._events.append(('time', a_datetime))
+        super(LoggingResult, self).time(a_datetime)
+
+
+def is_stack_hidden():
+    return TracebackContent.HIDE_INTERNAL_STACK
+
+
+def hide_testtools_stack(should_hide=True):
+    result = TracebackContent.HIDE_INTERNAL_STACK
+    TracebackContent.HIDE_INTERNAL_STACK = should_hide
+    return result
+
+
+def run_with_stack_hidden(should_hide, f, *args, **kwargs):
+    old_should_hide = hide_testtools_stack(should_hide)
+    try:
+        return f(*args, **kwargs)
+    finally:
+        hide_testtools_stack(old_should_hide)
+
+
+class FullStackRunTest(runtest.RunTest):
+
+    def _run_user(self, fn, *args, **kwargs):
+        return run_with_stack_hidden(
+            False,
+            super(FullStackRunTest, self)._run_user, fn, *args, **kwargs)
